<?php

require_once(mb_path('/lib-action/action.php'));

require_once(MB_LIBPATH.'/lib-tk/include/sso/util.php');
require_once(MB_LIBPATH.'/lib-tk/include/sso/post_binding.php');
require_once(MB_LIBPATH.'/lib-tk/include/sso/SAMLSign.php');
require_once(MB_LIBPATH.'/lib-tk/include/sso/Tokens.php');
require_once(MB_LIBPATH.'/lib-tk/include/sso/url.php');
require_once(MB_LIBPATH.'/lib-tk/include/sso/SAMLUtil.php');
require_once(MB_LIBPATH.'/lib-tk/include/sso/SSODelegatedUIRequest.php');

/**
 * This action processes requests from SSO server to this application.
 * Two types of requests should be processed:
 * 1. Credentials checking request
 * 2. Logout (single logout, SSOut, SLO) request and response.
 */
class ProcessSSOAPIRequest extends MB_Action {

    function ProcessSSOAPIRequest() {
        parent::MB_Action();
        $this->stringRep = __CLASS__;
    }

    function ssoCheckAdminCredentials($name, $password) {
        $h =& new QueryMaster('select', getDBCol('adminID'));
        $h->addTables('admins');
        $h->addAndClause(getDBCol('adminUsername'), '=', $name);
        if ($password) {
            $h->addAndClause(getDBCol('adminPassword'), '=', getPasswordHash($password));
        } else {
            // Only check that identifier is correct
        }
        $data = $h->getRow();
        if (count($data)) {
            return $data[getDBCol('adminID')];
        } else {
            return null;
        }
    }

    function ssoCheckClientCredentials($name, $password) {
        $h =& new QueryMaster('select', "clients.client_id");
//        $h->addTables('client_contacts');
//        $h->addAndClause(getDBCol('contactUsername'), '=', $name);
		$h->addJoin("clients", "client_id", "client_contacts");
        $h->addAndClause("client_contacts.contact_username", "=", $name);
		$h->addAndClause("client_contacts.contact_active", "=", 1, "i");
		$h->addAndClause("clients.client_active", "=", 1, "i");
		$h->addAndClause("clients.client_status", "=", getAPIStatusID("client", "active"), "i");

        if ($password) {
            $h->addAndClause("client_contacts.contact_password", "=", getPasswordHash($password));
        } else {
            // Only check that identifier is correct
        }
        $data = $h->getRow();
        if (count($data)) {
            return $name;
        } else {
            return null;
        }
    }

    function ssoCheckCredentialsHandler($impl, $args) {
        if ('on' !== getAt($_SERVER, 'HTTPS')) {
            throw new Exception('Forbidden. Only secure protocol (HTTPS) is allowed.', 403);
        }

        // TODO (security, sso) Cache origin ids (and requestor IPs) to mitigate DoSes and password brute force attacks. (memcached?)
        // $origin = getAt($args, 'request_origin');

        $user = getAt($args, 'user_name');
        $password = getAt($args, 'password');
        $userId = $impl->checkCredentials($user, $password);
        header('Content-Type: text/plain');
        echo $userId ? 'user_id = '.$user : '';
    }

    function processLogoutRequest($impl, saml_Tokens $samlTokens, $domXml, $relayState) {
        // IdP signature is already checked
        $samlProps = lib_Xml::queryProps(
            saml_Tokens::getXpath($domXml),
            array(
                'req_id' => '//@ID',
                'user_id' => '//saml:NameID',
                'idp_id' =>  '//saml:Issuer' ) );
        $impl->closeSessionByName($samlProps['user_id']);

        $idpId = $samlTokens->getIssuer($domXml);
        $ssoRegProps = SSOSettings::getProps();
        if ($ssoRegProps['idp_id'] != $idpId) {
            throw new Exception(
                'Cannot send logout response: unknown IdP identifier (signature should have been incorrect)',
                500 );
        }

		$destinationUrl = $ssoRegProps['api_url'] . '/SAML';
        SAMLUtil::sendMessage(
			$destinationUrl,
			$samlTokens->createLogOutResponse(
				$samlTokens->getID($domXml),
				$ssoRegProps['api_url'] . '/SAML',
				$ssoRegProps['sp_id']
			),
            $relayState,
            $ssoRegProps['sp_pkey'] );
    }

    function processLogoutResponse($impl, $relayState) {
        // Signature is already checked so we'll ignore parameters for now

        // If session is opened then do nothing (most likely this is a stale response),
        // otherwise show login screen:
        redirect('dashboard.php', getActionID('ShowDashboard'));
    }

    /**
     * Handle SAML message from IdP (SSO) server
     *
     * @param array $args
     */
    function processSAMLMessage($impl, $args) {
        $samlTokens = new saml_Tokens();
        $relayState = getAt($args, 'RelayState');

        $request = getAt($args, 'SAMLRequest');
        if (!is_null($request)) {
            $domXml = SAMLUtil::unpackMessage($request);
            if ($samlTokens->getMsgType($domXml) == saml_Tokens::MSG_LOGOUT_REQUEST) {
                $this->processLogoutRequest($impl, $samlTokens, $domXml, $relayState);
            } else {
                throw new Exception('Bad request (SAML message is not LogoutRequest)', 400);
            }
        } else {
            $request = getAt($args, 'SAMLResponse');
            if (!is_null($request)) {
                $domXml = SAMLUtil::unpackMessage($request);
                if ($samlTokens->getMsgType($domXml) == saml_Tokens::MSG_LOGOUT_RESPONSE) {
                    $this->processLogoutResponse($impl, $relayState);
                } else {
                    throw new Exception('Bad request (SAML message is not LogoutResponse)', 400);
                }
            } else {
               throw new Exception('Bad request (SAML message is not recognized)', 400);
            }
        }
    }

	function processUIMessage($args) {
        $uiType			= getAt($args, SSODelegatedUIRequest::POST_FIELD_UI_TYPE);
        $relayState		= getAt($args, SSODelegatedUIRequest::POST_FIELD_RELAY_STATE);
        $responseUrl	= getAt($args, SSODelegatedUIRequest::POST_FIELD_RESPONSE_URL);
        $requestId		= getAt($args, SSODelegatedUIRequest::POST_FIELD_REQUEST_ID);
        $messageId		= getAt($args, SSODelegatedUIRequest::POST_FIELD_MESSAGE_ID);

		$this->checkDelegatedUIType($uiType);
		if (! empty($messageId)) $this->checkDelegatedUIMessageId($messageId);

		$smartyVars = array();
		if (! empty($messageId)) {
			$smartyVars["message"] = 'TRANS_'.$messageId;
		}
		$smartyVars["relay_state"] = $relayState;
		$smartyVars["request_id"] = $requestId;
		$smartyVars["response_url"] = $responseUrl;

		$redir_url = $root_path."login.php?parentConfigGroupID=0&action=";

		switch ($uiType) {
			case SSODelegatedUIRequest::UI_TYPE_LOGIN:
				echo make_redirected_post_html($redir_url.getActionID('ShowIDPLogin'), $smartyVars, true);
				exit;
			case SSODelegatedUIRequest::UI_TYPE_SELECT_NAME:
				foreach (array_keys($args) as $key) {
					if (preg_match('/^(name_\d+)$/', $key, $matches)) {
						$smartyVars[$key] = $args[$key];
					}
				}
				echo make_redirected_post_html($redir_url.getActionID('ShowIDPSelectName'), $smartyVars, true);
				exit;
			case SSODelegatedUIRequest::UI_TYPE_FI_ATTACH:
				$smartyVars['fi_user_name_id'] = getAt($args, 'fi_user_name_id');
				$smartyVars['local_user_id'] = getAt($args, 'local_user_id');
				echo make_redirected_post_html($redir_url.getActionID('ShowIDPFIAttach'), $smartyVars, true);
				exit;
			case SSODelegatedUIRequest::UI_TYPE_FI_DETACH:
				$smartyVars['fi_user_name_id'] = getAt($args, 'fi_user_name_id');
				$smartyVars['local_user_id'] = getAt($args, 'local_user_id');
				echo make_redirected_post_html($redir_url.getActionID('ShowIDPFIConfirmDetach'), $smartyVars, true);
				exit;
			case SSODelegatedUIRequest::UI_TYPE_FI_UPDATE:
				$smartyVars['fi_user_name_id'] = getAt($args, 'fi_user_name_id');
				$smartyVars['fi_user_email'] = getAt($args, 'fi_user_email');
				echo make_redirected_post_html($redir_url.getActionID('ShowIDPFIUpdate'), $smartyVars, true);
				exit;
			case SSODelegatedUIRequest::UI_TYPE_FI_DELETE:
				$smartyVars['fi_user_name_id'] = getAt($args, 'fi_user_name_id');
				echo make_redirected_post_html($redir_url.getActionID('ShowIDPFIConfirmDelete'), $smartyVars, true);
				exit;
		}

		echo nl2br('<pre>'.var_export($args, 1).'</pre>');
		return false;
	}

    function getImplementation() {
        // XXX (refactoring) (I still do not like it but it is better than having these switches everywhere. /Petr)
        switch (APP_CURRENT) {
            case 'app-modernbill-admin':
                require_once(MB_LIBPATH.'/app-modernbill-admin/include/lib-action/sso/AdminSSOAPI.php');
                return new AdminSSOAPI();

            case 'app-modernbill-client':
                require_once(MB_LIBPATH.'/app-modernbill-client/include/lib-action/sso/ClientSSOAPI.php');
                return new ClientSSOAPI();

            default:
                throw new Exception('Can not initialise: current program component is unknown ('
                    . var_export(APP_CURRENT, true) . ')');
        }
    }

    /**
     * The main method of this class that executes the ShowLogin action, and returns success or failure.
     *
     * @return  int  1 if this action executed successfully, or 0 if otherwise.
     */
    function execute() {
        parent::execute();
        $impl = $this->getImplementation();
        try {
            $path = $_GET['sso_api_path'];
            switch (strtolower($path)) {
                case '/users':
                    $this->ssoCheckCredentialsHandler($impl, array_merge($_GET, $_POST));
                    break;

                case '/saml':
                case '/signout':
                    $this->processSAMLMessage($impl, $_POST);
                    break;

				case '/ui':
                    $this->processUIMessage($_POST);
					break;

                default:
                    throw new Exception("Resource ($path) is not found.", 404);
            }
            return true;
        } catch (Exception $e) {
            header("HTTP/1.1 {$e->getCode()} Bad request");
            header('Content-Type: text/plain');
            echo $e->getMessage();
            return false;
        }
    }

	private function checkDelegatedUIType($uiType) {
		$all_ui_types = SSODelegatedUIRequest::getSupportedUITypes();
		if (! in_array($uiType, $all_ui_types)) {
            $path = $_GET['sso_api_path'];
			throw new Exception("Resource ($path): Unknown ui_type \"$uiType\" has been passed.", 400);
		}
	}

	private function checkDelegatedUIMessageId($messageId) {
		$all_message_ids = SSODelegatedUIRequest::getSupportedMessageIds();
		if (! in_array($messageId, $all_message_ids)) {
            $path = $_GET['sso_api_path'];
			throw new Exception("Resource ($path): Unknown message_id \"$messageId\" has been passed.", 400);
		}
	}
} // ProcessSSOAPIRequest

?>