<?php

/////////////////////////////////////////////
// to add an upgrade path, add the version to
// the versions array in the constructor and
// then add the upgrade function to perform
// the needed actions.  The function name 
// needs to be the same as the version except
// the dots (.) and hyphens (-) need to be
// replaced with underscores (_) and be
// prefixed with "upgrade_".
// example: 	version: 5.6.0-1
// 		function: upgrade_5_6_0_1
// the function needs to add all error 
// messages to the internal errors array.
// example: $this->errors[] = "message here";
// and needs to return false on error, true
// on success.
/////////////////////////////////////////////


set_time_limit(0);
ignore_user_abort();

class PPBUpgrader {

	function PPBUpgrader($installVersion, $filesetVersion){
		$this->installVersion = $installVersion;
		$this->filesetVersion = $filesetVersion;
		$this->log = array("Upgrader Initialized.");
		$this->versions = array(
			"5.3.0-0",
			"5.3.2-1",
			"5.6.0-1",
			"6.0.0-1",
		);
	}

	function upgrade(){

		foreach ($this->versions as $v){
			if (version_compare($v, $this->installVersion, ">")){
				$f = "upgrade_".str_replace(array(".","-"), "_", $v);
				$r = $this->$f();
				if (!$r){
					return false;
				} else {
					$this->setNewVersion($v);
				}
			}
		}
		return true;
	}

	function upgrade_5_3_0_0(){
		// this upgrade path should NEVER be run and if it is then we have a version older
		// than this upgrade system can handle.
		$this->errors[] = "Sorry, the version you are using must be upgraded through our standalone installer before being used with Parallels Plesk Panel Licensing.";
		return false; //false;
	}

	function upgrade_5_3_2_1(){
		$tldID = $GLOBALS["db"]->GetOne("SELECT tld_id FROM tlds WHERE tld_extension = 'me'");
		if (!$tldID){
			$this->runQuery("INSERT INTO tlds (registrar_id, tld_extension, tld_name, tld_whois_server, tld_whois_resp, tld_active, tld_auto_search, tld_transfer_minimum_cycle_id, tld_minimum_cycle_id, tld_maximum_cycle_id, tld_renew_type, tld_is_transferable, tld_transfer_type, tld_idn_support, tld_idn_type, tld_remote_lock_enabled, tld_display_order) values (44, 'me', 'me', 'whois.meregistry.net', '', 1, 1, 7, 7, 15, 2, 1, 0, 1, 0, 0, 0)");
		}
		return true;
	}

	function upgrade_5_6_0_1(){
		// don't need to do anything here.
		return true;
	}

	function upgrade_6_0_0_1(){
		$this->runQuery("INSERT INTO admin_group_action_rel VALUES (1, 'InitSSOLogin'),(1, 'InitSSOLogout'),(1, 'ProcessSSOAPIRequest'),(1, 'ReceiveSSOLoginStatus'),(1, 'SelectSSOLoginMethod'),(1, 'ShowIDPFIAttach'),(1, 'ShowIDPFIConfirmDelete'),(1, 'ShowIDPFIConfirmDetach'),(1, 'ShowIDPFIManagement'),(1, 'ShowIDPFIUpdate'),(1, 'ShowIDPLogin'),(1, 'ShowIDPSelectName')");
		$this->runQuery("UPDATE config_groups SET config_group_name = 'pleskunix', config_group_trans_key = 'TRANS_PLESKUNIX' WHERE config_group_name = 'plesk'");
		$this->runQuery("UPDATE config_groups SET config_group_trans_key = 'PleskUnix' WHERE config_group_trans_key = 'Plesk'");
		return true;
	}

	function getErrors(){
		return (array)$this->errors;
	}

	function setNewVersion($version){
		$h =& new QueryMaster("update");
		$h->addTables("config_params");
		$h->addParam("config_param_value", $version, "s");
		$h->addAndClause("config_param_name", "=", "installVersion", "s");
		return $h->execute();
	}

	function runQuery($q){
		if ($s = $GLOBALS["db"]->Execute($q)){
			return (substr(trim(strtolower($q)), 0, 6)=="insert") ? $GLOBALS["db"]->Insert_ID() : true;
		}
		return false;
	}

}