<?php
// $Id: ProcessPanel.php 13216 2008-10-06 19:46:05Z kbailey $
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../lib-tk/shield.php";
require_once mb_path("/lib-mbapi/Command.php");
require_once mb_path("/lib-action/include/url/url.php");
require_once mb_path("/lib-tk/include/crypt/crypt.php");
require_once mb_path("/lib-mbapi/include/mail/mail.php");
require_once mb_path("/lib-modernbill/include/config/internal/cycles.php");
require_once mb_path("/lib-modernbill/include/config/internal/emailactions.php");


/**
 *
 * <mbapi>
 *   <command>ProcessPanelPackageSync</command> (REQUIRED)
 *   <showXMLHeader></showXMLHeader>
 *   <params>
 *     <configGroupID></configGroupID>
 *     <serverGroupID></serverGroupID>
 *   </params>
 * </mbapi>
 *
 * @package modernbill
 * @subpackage mbapi
 * @version 0.1
 *
 * @author  Kris Bailey
 */
class MBAPI_ProcessPanelPackageSyncCommand extends MBAPI_Command {

	/**
	 * Iterates through each panel in the in the system and for each one runs code to sync up packages for that
	 * panel with the panel if needed.  Initially only meant to get GUIDs for Plesk packages that were
	 * provisioned to a Plesk server that did not have GUID support when the Plesk server was upgraded later.
	 *
	 * @return  The result of the ProcessPanelPackageSync command based upon the given information.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		loadConfigs("server", "serverGroups", "panel");
		
		if ($CONF["server"]["serverGroups"]["panel"]["pleskunix"]){
			if ($CONF["server"]["serverGroups"]["panel"]["pleskunix"]["configGroupID"]){
				$CONF["server"]["serverGroups"]["panel"]["pleskunix"] = array($CONF["server"]["serverGroups"]["panel"]["pleskunix"]);
			}
			foreach ($CONF["server"]["serverGroups"]["panel"]["pleskunix"] as $x=>$params){
				if (!(int)$params["panelIDSynced"] || (int)$this->params['forceSync'][0]){
					if ($this->pleskHasGUID($params)){
					
						$serverGroup = oneOf(":serverGroupRels", array(
							'serverID' => $params['configGroupID'],
						));
					
						$eligiblePackages = $this->getPackagesForConfigGroupID($params["configGroupID"]);
						$guids = arrayFor(":packageAttributes", array(
							"packageAttributeName" => "guid",
							"packageID" => $eligiblePackages,
						));
						
						$eligiblePackages = array_flip($eligiblePackages);
						foreach ((array)$guids as $g){
							if ($g["packageAttributeValue"]!=''){
								unset($eligiblePackages[$g['packageID']]);
							} else {
								// blank guid package attribute, we need to remove it so we can only do inserts later and not updates
								deleteForKey(":packageAttributes", $g['packageAttributeID']);
							}
						}
						$eligiblePackages = array_keys($eligiblePackages);
						
						$h =& new QueryMaster("packages.primary_domain_id");
						$h->addColumns("packages.package_id");
						$h->addColumns("package_domains.domain_id");
						$h->addColumns("package_domains.domain_sld");
						$h->addColumns("package_domains.domain_tld");
						$h->addJoin("packages", "package_id", "package_domain_rel");
						$h->addJoin("package_domain_rel", "domain_id", "package_domains");
						$h->addAndClause("packages.package_id", "=", $eligiblePackages, "i");
						
						$packageDomains = $h->getResults();
						
						$packages = array();
						// put in the primary domain first
						foreach ($packageDomains as $pd){
							if ($pd['primary_domain_id']==$pd['domain_id']){
								$packages[$pd['package_id']][] = $pd['domain_sld'].".".$pd['domain_tld'];
							}
						}
						// ok, put in all the secondary domains
						foreach ($packageDomains as $pd){
							if ($pd['primary_domain_id']!=$pd['domain_id']){
								$packages[$pd['package_id']][] = $pd['domain_sld'].".".$pd['domain_tld'];
							}
						}
						
						foreach ($packages as $packageID=>$domains){
							foreach ($domains as $domainName){
								if ($guid = $this->getClientGuidFromDomainName($params, $domains)){
									insertFor(':packageAttributes', array(
										'serverGroupID' => $serverGroup['serverGroupID'],
										'configGroupID' => $params['configGroupID'],
										'packageAttributeName' => 'guid',
										'packageAttributeValue' => $guid,
										'packageAttributeDateCreated' => time(),
										'packageID' => $packageID,
									));
									$attr = oneOf(':packageAttributes', array(
										'packageID' => $packageID,
										'packageAttributeName' => 'masterAccountID',
										'configGroupID' => $params['configGroupID'],
									));
									// update the client id too since it could possibly be out of sync
									if ($attr['packageAttributeID']){
										updateForKey(':packageAttributes', $attr['packageAttributeID'], array(
											'packageAttributeValue' => $this->clientID,
										));
									} else {
										insertFor(':packageAttributes', array(
											'serverGroupID' => $serverGroup['serverGroupID'],
											'configGroupID' => $params['configGroupID'],
											'packageAttributeName' => 'masterAccountID',
											'packageAttributeValue' => $this->clientID,
											'packageAttributeDateCreated' => time(),
											'packageID' => $packageID,
										));
									}
									break;
								}
							}
						}
						insertFor(':configParams', array(
							'configParamName' =>  'panelIDSynced',
							'configGroupID' => $params['configGroupID'],
							'configParamValue' => 1,
							'configParamInputType' => 'h',
						));
					}
				}
			}
		}
		
		if ($CONF["server"]["serverGroups"]["panel"]["pleskwin"]){
			if ($CONF["server"]["serverGroups"]["panel"]["pleskwin"]["configGroupID"]){
				$CONF["server"]["serverGroups"]["panel"]["pleskwin"] = array($CONF["server"]["serverGroups"]["panel"]["pleskwin"]);
			}
			foreach ($CONF["server"]["serverGroups"]["panel"]["pleskwin"] as $x=>$params){
				if (!(int)$params["panelIDSynced"] || (int)$this->params['forceSync'][0]){
					if ($this->pleskHasGUID($params)){
					
						$serverGroup = oneOf(":serverGroupRels", array(
							'serverID' => $params['configGroupID'],
						));
					
						$eligiblePackages = $this->getPackagesForConfigGroupID($params["configGroupID"]);
						$guids = arrayFor(":packageAttributes", array(
							"packageAttributeName" => "guid",
							"packageID" => $eligiblePackages,
						));
						
						$eligiblePackages = array_flip($eligiblePackages);
						foreach ((array)$guids as $g){
							if ($g["packageAttributeValue"]!=''){
								unset($eligiblePackages[$g['packageID']]);
							} else {
								// blank guid package attribute, we need to remove it so we can only do inserts later and not updates
								deleteForKey(":packageAttributes", $g['packageAttributeID']);
							}
						}
						$eligiblePackages = array_keys($eligiblePackages);
						
						$h =& new QueryMaster("packages.primary_domain_id");
						$h->addColumns("packages.package_id");
						$h->addColumns("package_domains.domain_id");
						$h->addColumns("package_domains.domain_sld");
						$h->addColumns("package_domains.domain_tld");
						$h->addJoin("packages", "package_id", "package_domain_rel");
						$h->addJoin("package_domain_rel", "domain_id", "package_domains");
						$h->addAndClause("packages.package_id", "=", $eligiblePackages, "i");
						
						$packageDomains = $h->getResults();
						
						$packages = array();
						// put in the primary domain first
						foreach ($packageDomains as $pd){
							if ($pd['primary_domain_id']==$pd['domain_id']){
								$packages[$pd['package_id']][] = $pd['domain_sld'].".".$pd['domain_tld'];
							}
						}
						// ok, put in all the secondary domains
						foreach ($packageDomains as $pd){
							if ($pd['primary_domain_id']!=$pd['domain_id']){
								$packages[$pd['package_id']][] = $pd['domain_sld'].".".$pd['domain_tld'];
							}
						}
						
						foreach ($packages as $packageID=>$domains){
							foreach ($domains as $domainName){
								if ($guid = $this->getClientGuidFromDomainName($params, $domains)){
									insertFor(':packageAttributes', array(
										'serverGroupID' => $serverGroup['serverGroupID'],
										'configGroupID' => $params['configGroupID'],
										'packageAttributeName' => 'guid',
										'packageAttributeValue' => $guid,
										'packageAttributeDateCreated' => time(),
										'packageID' => $packageID,
									));
									$attr = oneOf(':packageAttributes', array(
										'packageID' => $packageID,
										'packageAttributeName' => 'masterAccountID',
										'configGroupID' => $params['configGroupID'],
									));
									// update the client id too since it could possibly be out of sync
									if ($attr['packageAttributeID']){
										updateForKey(':packageAttributes', $attr['packageAttributeID'], array(
											'packageAttributeValue' => $this->clientID,
										));
									} else {
										insertFor(':packageAttributes', array(
											'serverGroupID' => $serverGroup['serverGroupID'],
											'configGroupID' => $params['configGroupID'],
											'packageAttributeName' => 'masterAccountID',
											'packageAttributeValue' => $this->clientID,
											'packageAttributeDateCreated' => time(),
											'packageID' => $packageID,
										));
									}
									break;
								}
							}
						}
						insertFor(':configParams', array(
							'configParamName' =>  'panelIDSynced',
							'configGroupID' => $params['configGroupID'],
							'configParamValue' => 1,
							'configParamInputType' => 'h',
						));
					}
				}
			}
		}
		
		$this->setXMLData('');
		return $this->getXMLData();

	}
	
	function getPackagesForConfigGroupID($id){
		$h =& new QueryMaster("packages.package_id");
		$h->addJoin("packages", "package_id", "package_attributes");
		$h->addAndClause("package_attributes.config_group_id", "=", $id, "i");
		$h->addGroupByColumns("packages.package_id");
		return (array)$h->getCol();
	}
	
	// function that returns true/false for whether a plesk box has GUIDs or not
	function pleskHasGUID($params){
		global $CONF;
		if ($this->pleskHasGUIDCache[$params["configGroupID"]]){
			return $this->pleskHasGUIDCache[$params["configGroupID"]];
		}

		$pleskUser = unsecureInput($params["moduleUsername"]);
		$pleskReseller = $params["pleskReseller"];

		$data ="<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?><packet></packet>";
		$result = $this->makePleskCall($params, $data);

		$d = array();
		if (is_array($result) && $result["packet"][0]["system"][0]["status"][0]=="error"){
			preg_match("/[1-9]\.[[:digit:]]\.[[:digit:]]\.[[:digit:]]/", $result["packet"][0]["system"][0]["errtext"][0], $d);
		}

		if (version_compare($d[0], '1.6.0.0', '>=')){ // has GUIDs
			return $this->pleskHasGUIDCache[$params["configGroupID"]] = true;
		} else {
			return $this->pleskHasGUIDCache[$params["configGroupID"]] = false;
		}
	}
	
	/*
	* method to make a call to plesk and return the contents
	*/
	function makePleskCall($params, $xml){
		global $CONF;
		$xml = str_replace(array("\n","\t", "  "),"",$xml);
		dbg("+++ PLESK SERVER CALL +++");

		$url = $params["api"];
		if (substr($url, -1)=="/") $url = substr($url, 0, -1);
		$pleskUser = unsecureInput($params["moduleUsername"]);
		$pleskPass = getStuff(unsecureInput($params["modulePassword"]));
		$url = $url."/enterprise/control/agent.php";

		// Create the curl object
		$c =& new Curl();
		$c->addField("",$xml,0);

		// Call the cURL class and execute the call
		$c->setURL($url);
		$c->addHeader("HTTP_AUTH_LOGIN: " . $pleskUser);
		$c->addHeader("HTTP_AUTH_PASSWD: " . $pleskPass);
		$c->addHeader("HTTP_PRETTY_PRINT: TRUE");
		$c->addHeader("Content-Type: text/xml");
	
		$result = $c->toString();
		$this->connectionInfo = $c->connectionInfo;
		$x =& new MBXMLLite($result);
		return $x->toArray();
	}

	function getClientGuidFromDomainName($params, $domainName){
		if (!$domainName) return false;
		$result = collapseArrayElements($this->makePleskCall($params, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>
		<packet version=\"1.6.0.0\">".toXML(array(
			"domain" => array(
				"get" => array(
					"filter" => array(
						"domain-name" => $domainName,
					),
					"dataset" => array(
						"gen_info" => array(),
					),
				),
			),
		))."</packet>"));
		if (is_array($result) && $result["packet"]["domain"]["get"]["result"]["status"]=="ok"){
			$result = collapseArrayElements($this->makePleskCall($params, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>
			<packet version=\"1.5.2.0\">".toXML(array(
				"client" => array(
					"get" => array(
						"filter" => array(
							"id" => $result["packet"]["domain"]["get"]["result"]["data"]["gen_info"]["owner-id"],
						),
						"dataset" => array(
							"gen_info" => array(),
						),
					),
				),
			))."</packet>"));
			if (is_array($result) && $result["packet"]["client"]["get"]["result"]["status"]=="ok"){
				$this->clientID = $result["packet"]["domain"]["get"]["result"]["data"]["gen_info"]["owner-id"];
				return $result["packet"]["client"]["get"]["result"]["data"]["gen_info"]["guid"];
			} else {
				dbg("GUID call result not ok.");
				return '';
			}

		} else {
			dbg("client id call result not ok.");
			return '';
		}
	}


}